VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsIniFile"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
'-----------------------------------------------
'
'  clsIniFile  Class zur Verwaltung von IniFiles
'
'  Autor:         peter.k.sauer@web.de
'  Created:       08.02.2004
'  Update:        09.02.2004
'-----------------------------------------------

'----------------------    wesentliche Methoden
'  IniFileRead             IniDatei laden
'  IniFileWrite            IniDatei schreiben

'  GetProfile              einen Eintrag lesen
'  WriteProfile            einen Eintrag schreiben
'  WriteSection            eine Section anlegen/Kommentar updaten

'----------------------    weitere Methoden
'  ExistIniFile            existiert IniFile
'  ExistSection            existiert eine Section
'  ExistProfile            existiert ein Eintrag
'  RenameSection           eine Section umbenennen
'  RenameProfile           ein Profile (Key) umbenennen
'  RemoveSection           eine Section lschen
'  RemoveProfile           ein Profile (Key) lschen

'  GetProfileCount         Anzahl Eintrge einer Section
'  GetSectionComments      Kommentar zu einer Section lesen
'  GetSectionNames         alle Section Bezeichnungen lesen
'  GetSectionProfileKeys   alle Keys zu Profiles einer Section

'----------------------    Eigenschaften
'  Changed                 wurden Eintrge gendert
'  AutoSave(Default=False) automatisch IniFileWrite bei Terminate
'                          Default=False
'  CommentChar             Zeichen fr Kommentar, Default ;
'  IniFileComments         Kommentare (im Kopf) der IniFile
'  SectionsCount           Anzahl der Sections
'  IniFileString           Inhalt der IniFile als String
'----------------------------------------------------------------

Option Explicit

Private Type ProfileType               'ein einzelnes Profile
   Key As String                       'Bezeichnung
   Value As String                     'Inhalt
   Comment As String                   'Kommentar
End Type

Private Type SectionType               'eine Section
   Name As String                      'Bezeichnung
   Comments As String                  'Kommentar
   Rows() As ProfileType               'Collection der Profiles
End Type

Private SectionsCol() As SectionType   'Collection der Sections

Private mvarAutoSave As Boolean        'automatische Sicherung Ini
Private mvarCommentChar As String      'Trennzeichen fr Kommentare
Private mvarIniFileName As String      'Name der IniFile
Private mvarSectionsCount As Long      'Anzahl der Sections
Private mvarIniFileComments As String  'Kommentare am Anfang der IniFile
Private mvarIniFileString As String    'Inhalt der IniFile nach dem Lesen
Private mvarChanged As Boolean         'wurden nderungen gemacht

Public Property Get Changed() As Boolean
    Changed = mvarChanged
End Property

Public Property Get IniFileString() As String
    IniFileString = mvarIniFileString
End Property

Public Property Let IniFileComments(ByVal vData As String)
    mvarIniFileComments = vData
End Property

Public Property Get IniFileComments() As String
    IniFileComments = mvarIniFileComments
End Property

Public Property Let SectionsCount(ByVal vData As Long)
    mvarSectionsCount = vData
End Property

Public Property Get SectionsCount() As Long
    SectionsCount = mvarSectionsCount
End Property

Public Property Get IniFileName() As String
    IniFileName = mvarIniFileName
End Property

Public Property Let AutoSave(ByVal vData As Boolean)
    mvarAutoSave = vData
End Property

Public Property Get AutoSave() As Boolean
    AutoSave = mvarAutoSave
End Property

Public Property Let CommentChar(ByVal vData As String)
    mvarCommentChar = vData
End Property

Public Property Get CommentChar() As String
    CommentChar = mvarCommentChar
End Property

'existiert die IniFile
Public Function ExistIniFile(Path As String) As Boolean

      If Len(Dir(Path)) > 0 Then
         ExistIniFile = True
      End If
End Function

'eine IniFile einlesen
Public Function IniFileRead(Path As String, _
                            Optional ErrNumber As Long = 0, _
                            Optional ErrDescription As String = "") As Boolean

   Dim FNr As Integer
   Dim s() As String
   Dim i As Long
   
      'Dateinamen saven
      mvarIniFileName = Path
   
      'vorhandene Teile killen
      ReDim SectionsCol(0)
      SectionsCount = 0
      mvarIniFileComments = ""
      mvarIniFileString = ""
      
      'Datei vorhanden ??
      If Len(Dir(Path)) = 0 Then
         ErrNumber = 53
         ErrDescription = "Datei nicht gefunden"
         Exit Function
      End If
      
      'IniFile einlesen
      FNr = FreeFile
      Open Path For Binary As #FNr
      mvarIniFileString = Space(LOF(FNr))
      Get #FNr, , mvarIniFileString
      Close #FNr
      
      'das Teil ist leer
      If Len(mvarIniFileString) = 0 Then
         IniFileRead = True
         Exit Function
      End If
      
      'zerlegen in Zeilen
      s() = Split(mvarIniFileString, vbCrLf)
      
      'Kommentare am Anfang filtern
      For i = 0 To UBound(s)
         If Len(LTrim(s(i))) > 0 Then
            If Left$(s(i), Len(CommentChar)) <> CommentChar Then
               Exit For
            End If
            mvarIniFileComments = mvarIniFileComments & s(i) & vbCrLf
         End If
      Next
      
      'Sections anlegen
      SectionsCreateRead s()
      
      IniFileRead = True
End Function

'Sections aufnehmen in Collection/Array
Private Sub SectionsCreateRead(Rows() As String)

   Dim i As Long
   Dim s As String
   
      For i = LBound(Rows) To UBound(Rows)
         s = Trim(Rows(i))
         If Left$(s, 1) = "[" Then
            If Right$(s, 1) = "]" Then
               'Section gefunden, einlesen
               SectionAddRead Mid$(s, 2, Len(s) - 2), Rows(), i + 1
            End If
         End If
      Next
End Sub
   
'eine Section aufnehmen in Collection/Array
Private Sub SectionAddRead(SectionName, Rows() As String, Row As Long)

   Dim Section As SectionType
   Dim s As String
   Dim i As Long, j As Long, k As Long
   Dim V As Variant
   
      With Section
         .Name = SectionName
         ReDim .Rows(0)
         
         For i = Row To UBound(Rows)
            s = LTrim(Rows(i))
            'neue Section
            If Left$(s, 1) = "[" Then
               Exit For
            
            'Kommentar zur Section
            ElseIf Left$(s, Len(CommentChar)) = CommentChar Then
               If Len(.Comments) > 0 Then
                  .Comments = .Comments & vbCrLf & s
               Else
                  .Comments = s
               End If
               
            'Check auf Profile
            ElseIf Len(s) > 0 Then
               k = InStr(1, s, "=")
               If k > 0 Then
                  'neues Profile
                  j = UBound(.Rows) + 1
                  ReDim Preserve .Rows(j)
                  .Rows(j).Key = Left$(s, k - 1)
                  .Rows(j).Value = Mid$(s, k + 1)
                  
                  'Kommentar rausfiltern
                  k = InStr(1, .Rows(j).Value, CommentChar)
                  If k > 0 Then
                     .Rows(j).Comment = Mid$(.Rows(j).Value, k + Len(CommentChar))
                     .Rows(j).Value = Left$(.Rows(j).Value, k - 1)
                  End If
               End If
            End If
         Next
         'aufnehmen in Collection (Array)
         i = UBound(SectionsCol) + 1
         ReDim Preserve SectionsCol(i)
         SectionsCol(i) = Section
         SectionsCount = SectionsCount + 1
      End With
End Sub
   
'liefert den Index einer Section
Private Function GetSectionIndex(SectionName As String) As Long

   Dim i As Long
   
      For i = 1 To UBound(SectionsCol)
         If UCase(SectionsCol(i).Name) = UCase(SectionName) Then
            GetSectionIndex = i
            Exit Function
         End If
      Next
End Function
   
'existiert eine Section
Public Function ExistSection(SectionName As String) As Boolean

      If GetSectionIndex(SectionName) > 0 Then
         ExistSection = True
      End If
End Function

'liefert die Namen aller Sections
Public Function GetSectionsNames(Optional lbxSections As ListBox) As String()

   Dim i As Long
   Dim s() As String
         
      ReDim s(0)
      If SectionsCount > 0 Then
         ReDim s(SectionsCount - 1)
      End If
      
      'Listbox lschen
      If Not lbxSections Is Nothing Then
         lbxSections.Clear
      End If
      
      'Array fllen
      For i = 1 To UBound(SectionsCol)
         s(i - 1) = SectionsCol(i).Name
         If Not lbxSections Is Nothing Then
            lbxSections.AddItem SectionsCol(i).Name
         End If
      Next
      GetSectionsNames = s()
End Function
  
'liefert Kommentare zu einer Section
Public Function GetSectionComments(SectionName As String) As String

   Dim i As Long
   
      i = GetSectionIndex(SectionName)
      If i > 0 Then
         GetSectionComments = SectionsCol(i).Comments
      End If
End Function

  
'liefert die Keys aller Profiles einer Section
Public Function GetSectionProfileKeys(SectionName As String, _
                                     Optional lbxProfileKeys _
                                     As ListBox) As String()

   Dim i As Long, j As Long
   Dim s() As String
   
      ReDim s(0)
      If Not lbxProfileKeys Is Nothing Then
         lbxProfileKeys.Clear
      End If
   
      'Section suchen
      i = GetSectionIndex(SectionName)
      If i > 0 Then
         With SectionsCol(i)
            ReDim s(UBound(.Rows) - 1)
            For j = 1 To UBound(.Rows)
               s(j - 1) = .Rows(j).Key
               If Not lbxProfileKeys Is Nothing Then
                  lbxProfileKeys.AddItem .Rows(j).Key
               End If
            Next
         End With
      End If
End Function
   
'eine Section umbenennen
Public Function RenameSection(OldName As String, NewName As String) As Boolean

   Dim i As Long
   
      i = GetSectionIndex(OldName)
      If i = 0 Then
         'nicht gefunden
         Exit Function
      End If
      
      SectionsCol(i).Name = NewName
      mvarChanged = True
      RenameSection = True
End Function

'eine Section komplett lschen
Public Function RemoveSection(SectionName As String) As Boolean

   Dim i As Long, j As Long
   
      i = GetSectionIndex(SectionName)
      If i = 0 Then
         'nicht gefunden
         Exit Function
      End If
      
      'ab Section alle Elemente um 1 vorschieben
      For j = i To UBound(SectionsCol) - 1
         SectionsCol(j) = SectionsCol(j + 1)
      Next
      'letztes Element lschen
      j = UBound(SectionsCol) - 1
      ReDim Preserve SectionsCol(j)
      
      mvarChanged = True
      RemoveSection = True
End Function

   
'liefert den Index eines Profiles/Key
Private Function GetProfileIndex(SectionIndex As Long, Key As String) As Long

   Dim i As Long
   
      With SectionsCol(SectionIndex)
         For i = 1 To UBound(.Rows)
            If UCase(.Rows(i).Key) = UCase(Key) Then
               GetProfileIndex = i
               Exit For
            End If
         Next
      End With
End Function

   
'berprft ob ein Eintrag existiert
Public Function ExistProfile(SectionName As String, Key As String) As Boolean

   Dim i As Long, j As Long
   
      i = GetSectionIndex(SectionName)
      If i > 0 Then
         If GetProfileIndex(i, Key) > 0 Then
            ExistProfile = True
         End If
      End If
End Function
   
'liefert Value eines Eintrags
Public Function GetProfile(SectionName As String, Key As String, _
                           Optional Comment As String = "", _
                           Optional Default As String = "") As String
                                       
   Dim i As Long, j As Long
   
      i = GetSectionIndex(SectionName)
      If i > 0 Then
         j = GetProfileIndex(i, Key)
         If j > 0 Then
            Comment = SectionsCol(i).Rows(j).Comment
            GetProfile = SectionsCol(i).Rows(j).Value
            Exit Function
         End If
      End If
   
      'nicht gefunden, Default Wert bergeben
      GetProfile = Default
End Function

'eine Section (neu) schreiben
Public Function WriteSection(SectionName As String, _
                             Optional Comments As String = "", _
                             Optional UpdateIfExist As Boolean = True) As Boolean
                             
   Dim Section As SectionType
   Dim i As Long
   Dim s() As String
   
      s = Split(Comments, vbCrLf)
      'berprfen auf Kommentarzeichen, notfall korrigieren
      For i = LBound(s) To UBound(s)
         If Len(LTrim$(s(i))) > 0 Then
            If Left$(LTrim$(s(i)), Len(CommentChar)) <> CommentChar Then
               s(i) = CommentChar & s(i)
            End If
         End If
      Next
   
      'Update Kommentar falls Section vorhanden
      If ExistSection(SectionName) Then
         For i = 1 To UBound(SectionsCol)
            If UCase(SectionsCol(i).Name) = UCase(SectionName) Then
               SectionsCol(i).Comments = Join(s(), vbCrLf)
               WriteSection = True
               Exit Function
            End If
         Next
      End If
      
      'Section anfgen
      i = UBound(SectionsCol) + 1
      ReDim Preserve SectionsCol(i)
      With SectionsCol(i)
         .Name = SectionName
         .Comments = Join(s(), vbCrLf)
         ReDim .Rows(0)
      End With
      
      'nderung festhalten
      mvarChanged = True
      WriteSection = True
End Function
   
'Anzahl der Profiles einer Section ermitteln
Public Function GetProfileCount(SectionName As String) As Long

   Dim i As Long
   
      i = GetSectionIndex(SectionName)
      If i > 0 Then
         GetProfileCount = UBound(SectionsCol(i).Rows)
      Else
         'Section nicht gefunden
         GetProfileCount = -1
      End If
End Function
   
   
'schreibt einen Eintrag
Public Function WriteProfile(SectionName As String, _
                             Key As String, Value As String, _
                             Optional Comment As String, _
                             Optional AddSectionIfMissing As Boolean = True, _
                             Optional AddProfileIfMissing As Boolean = True) As Boolean

   Dim i As Long, j As Long
   
      i = GetSectionIndex(SectionName)
      'wenn Section nicht gefunden
      If i = 0 Then
         If Not AddSectionIfMissing Then
            Exit Function
         End If
         'Section anlegen
         WriteSection SectionName
         i = UBound(SectionsCol)
      End If

      j = GetProfileIndex(i, Key)
      'wenn Profile nicht gefunden
      If j = 0 Then
         If Not AddProfileIfMissing Then
            Exit Function
         End If
         'Profile anlegen
         j = UBound(SectionsCol(i).Rows) + 1
         ReDim Preserve SectionsCol(i).Rows(j)
      End If
      
      'eintragen Value
      With SectionsCol(i).Rows(j)
         .Key = Key
         .Value = Value
         .Comment = Comment
      End With
      
      'nderung feststellen
      mvarChanged = True
      WriteProfile = True
End Function
   
'ein Profile umbenennen
Public Function RenameProfile(SectionName As String, OldKey As String, _
                              NewKey As String) As Boolean

   Dim i As Long, j As Long
   
      i = GetSectionIndex(SectionName)
      If i = 0 Then
         'Section nicht gefunden
         Exit Function
      End If
      j = GetProfileIndex(i, OldKey)
      If j = 0 Then
         'Profile nicht gefunden
         Exit Function
      End If
      
      SectionsCol(i).Rows(j).Key = NewKey
      mvarChanged = True
      RenameProfile = True
End Function

'ein Profile lschen
Public Function RemoveProfile(SectionName As String, Key As String) As Boolean

   Dim i As Long, j As Long, k As Long
   
      i = GetSectionIndex(SectionName)
      If i = 0 Then
         'Section nicht gefunden
         Exit Function
      End If
      j = GetProfileIndex(i, Key)
      If j = 0 Then
         'Profile nicht gefunden
         Exit Function
      End If
      
      'ab Profile alle Elemente um 1 nach vorne schieben
      For k = j To UBound(SectionsCol(i).Rows) - 1
         SectionsCol(i).Rows(k) = SectionsCol(i).Rows(k + 1)
      Next
      
      'letztes Element lschen
      k = UBound(SectionsCol(i).Rows) - 1
      ReDim Preserve SectionsCol(i).Rows(k)
      
      mvarChanged = True
      RemoveProfile = True
End Function

'IniDatei schreiben
Public Function IniFileWrite(Optional ErrNumber As Long, _
                             Optional ErrDescription As String) As Boolean

   Dim s() As String
   Dim i As Long, j As Long
   Dim FNr As Integer
   
      On Error GoTo Fehler
   
      'IniFile ffnen
      FNr = FreeFile
      Open IniFileName For Output As #FNr
      
      'schreiben der Kommentare
      If Len(IniFileComments) > 0 Then
         Print #FNr, IniFileComments
      End If
      
      'schreiben der Sections
      For i = 1 To UBound(SectionsCol)
         With SectionsCol(i)
            'schreiben Sectionname
            Print #FNr, "[" & .Name & "]"
            
            'Kommentar zur Section schreiben
            If Len(.Comments) > 0 Then
               Print #FNr, .Comments
            End If
            
            'schreiben der Profiles
            For j = 1 To UBound(.Rows)
               If Len(.Rows(j).Comment) > 0 Then
                  Print #FNr, .Rows(j).Key & "=" & _
                              .Rows(j).Value & CommentChar & _
                              .Rows(j).Comment
               Else
                  Print #FNr, .Rows(j).Key & "=" & _
                              .Rows(j).Value
               End If
            Next
            
            '1 Leezeile einfgen
            If i < UBound(SectionsCol) Then
               Print #FNr, ""
            End If
         End With
      Next
      
      Close #FNr
      IniFileWrite = True
      
      'nderungen eleminieren
      mvarChanged = False
      Exit Function
      
Fehler:
      ErrNumber = Err.Number
      ErrDescription = Err.Description
End Function
   
'Ini der Class
Private Sub Class_Initialize()

      ReDim SectionsCol(0)
      mvarCommentChar = ";"
End Sub

'beim Beenden
Private Sub Class_Terminate()

      'wurden nderungen durchgefhrt
      If Changed Then
         If AutoSave Then
            IniFileWrite
            mvarChanged = False
         End If
      End If
End Sub
